//
//  ContributorCellView.swift
//  BookPlayer
//
//  Created by Gianni Carlo on 2/22/19.
//  Copyright © 2019 Tortuga Power. All rights reserved.
//

import UIKit

class ContributorCellView: UICollectionViewCell {
    @IBOutlet weak var imageView: UIImageView!

    override func awakeFromNib() {
        super.awakeFromNib()
    }
}

class CollectionViewRow {
    var attributes = [UICollectionViewLayoutAttributes]()
    var spacing: CGFloat = 0

    init(spacing: CGFloat) {
        self.spacing = spacing
    }

    func add(attribute: UICollectionViewLayoutAttributes) {
        self.attributes.append(attribute)
    }

    var rowWidth: CGFloat {
        return self.attributes.reduce(0) { result, attribute -> CGFloat in
            result + attribute.frame.width
        } + CGFloat(self.attributes.count - 1) * self.spacing
    }

    func centerLayout(collectionViewWidth: CGFloat) {
        let padding = (collectionViewWidth - self.rowWidth) / 2
        var offset = padding
        for attribute in self.attributes {
            attribute.frame.origin.x = offset
            offset += attribute.frame.width + self.spacing
        }
    }
}

class UICollectionViewCenterLayout: UICollectionViewFlowLayout {
    override func layoutAttributesForElements(in rect: CGRect) -> [UICollectionViewLayoutAttributes]? {
        guard let attributes = super.layoutAttributesForElements(in: rect) else {
            return nil
        }

        var rows = [CollectionViewRow]()
        var currentRowY: CGFloat = -1

        for attribute in attributes {
            if currentRowY != attribute.frame.origin.y {
                currentRowY = attribute.frame.origin.y
                rows.append(CollectionViewRow(spacing: 10))
            }
            rows.last?.add(attribute: attribute)
        }

        rows.forEach { $0.centerLayout(collectionViewWidth: collectionView?.frame.width ?? 0) }
        return rows.flatMap { $0.attributes }
    }
}
